#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;

using VVVV.Core.Logging;

using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.Net.NetworkInformation;
using System.Threading;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Ping",
	Category = "Network",
	Help = "Pings some ip addresses and informs you with the result",
	Tags = "",
	Warnings = " don't enter the same address twice!")]
	#endregion PluginInfo
	public class NetworkPingNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("IP", DefaultString = "192.168.100.1")]
		ISpread<string> FIp;
		
		[Input("Timeout", DefaultValue= 12000)]
		ISpread<int> FTimeout;
		
		[Input("Ping", IsBang=true)]
		ISpread<bool> FPing;
		
		[Output("Reachable")]
		ISpread<bool> FReachable;
		
		[Output("Status")]
		ISpread<string> FStatus;
		
		[Import()]
		ILogger FLogger;
		#endregion fields & pins
		
		
		public void Evaluate(int SpreadMax)
		{
			FStatus.SliceCount = SpreadMax;
			FReachable.SliceCount = SpreadMax;
			
			for (int i = 0; i < SpreadMax; i++)
			{
				if(FPing[i])
				{
					bool isValid = IsValidIP(FIp[i]);
					
					if (isValid == true)
					{
						FStatus[i] = "pinging ...";	//valid ip address
						
						Ping(FIp[i], FTimeout[i]);		// go and ping'em
					}else
					{
						FStatus[i] = "invalid ip address";
					}
				}
			}
		}
		
		#region methods
		public bool IsValidIP(string addr)
		{
			string pattern = "^([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.([01]?\\d\\d?|2[0-4]\\d|25[0-5])$";
			Regex check = new Regex(pattern);
			
			bool valid = false;
			
			if (addr == "")
			{
				valid = false;
				//FLogger.Log(LogType.Debug, "no address provided");
			}
			else
			{
				valid = check.IsMatch(addr, 0);
			}
			return valid;
		}
		
		private void Ping(string host, int timeout)
		{
			using (Ping p = new Ping())
			{
				try
				{
					//register Eventhandler
					p.PingCompleted += new PingCompletedEventHandler(p_PingCompleted);
					
					//AutoResetEvent waiter = new AutoResetEvent (false);
					
					string data = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
					byte[] buffer = Encoding.ASCII.GetBytes (data);
					
					PingOptions op = new PingOptions();
					//op.Ttl = 64;
					//op.DontFragment = true;
					
					p.SendAsync(host, timeout, buffer, op, host);
				}
				catch (Exception e)
				{
					FLogger.Log(LogType.Debug, "Exception: " + e.Message);
				}
			}
		}
		
		private void p_PingCompleted(object sender, System.Net.NetworkInformation.PingCompletedEventArgs e)
		{
			try
			{
				// get index of current reply
				int index = FIp.IndexOf(e.UserState.ToString());
				
				//FLogger.Log(LogType.Debug, "Ping for '{0}'  is '{1}' rtt: '{2}'",	e.UserState.ToString(),e.Reply.Status, e.Reply.RoundtripTime);
				//FLogger.Log(LogType.Debug, "index for " + e.UserState + " is " + index);
				
				if (e.Reply != null)
				{
					if (e.Reply.Status.ToString() == "Success")
					{
						FReachable[index] = true;
					}else{
						FReachable[index] = false;
					}
					
					//FStatus[index] = ("Ping for " + e.UserState + " is " + e.Reply.Status);
					FStatus[index] = e.Reply.Status.ToString();
				}else
				{
					FStatus[index] = " no reply";
				}
			}
			catch (Exception ex)
			{
				FLogger.Log(LogType.Debug, "Exception: " + IPStatus.Unknown + ex);
			}
		}
		#endregion methods
	}
}
